/*
 * @author Robert Koenighofer, member of the ISAC-team, 
 * Copyright (c) 2006 by Robert Koenighofer
 * created Jul 11, 2006 10:09:13 AM
 * Institute for Softwaretechnology, Graz University of Technology, Austria.
 * 
 * Use is subject to PGPL license terms.
 */

package isac.users;

import java.io.Serializable;
import java.rmi.RemoteException;
import java.rmi.server.UnicastRemoteObject;
import java.util.HashMap;

import org.apache.log4j.Logger;

import isac.browserdialog.IBrowserDialog;
import isac.session.SessionManager;
import isac.session.Session;

import isac.util.ObjectManagerPaths;
import isac.wsdialog.WSDialogManager;

/**
 * This class administrates all users. It is implemented as Singleton
 * (design Pattern of Gamma et.al.), which means, that there can be only
 * one instance of this class. This instance can be accessed by the
 * getInstance() method.
 * 
 * @author Robert Koenighofer
 */
public class UserManager extends UnicastRemoteObject implements IUserManager,
		Serializable {
	
	static final long serialVersionUID = 67389427;//FIXXME generate with plugin

	private static UserManager instance_;
	
	private HashMap<String,User> username_map_user_;
	
	private static final Logger logger = Logger.getLogger(UserManager.class.getName());

	/**
	 * The constructor is private, since this class is implemented
	 * as singleton. Use the getInstance() method to get the one and
	 * only instance of this class.
	 * 
	 * @throws RemoteException if remote object is not available any more
	 */
	private UserManager() throws RemoteException {
		username_map_user_ = new HashMap<String,User>();
	}

	/**
	 * Checks by use of {@link isac.users.Accounts} if username and 
	 * password is correct. If this is true an new Session is created by 
	 * use of the {@link isac.session.SessionManager}. The
	 * created Session object is returned. If the login fails, null
	 * is returned.
	 * 
	 * @param username the name of the user who wants to login
	 * @param pass the password the user has entered
	 * @return the Session which was created for the user or null if login fails
	 * @throws RemoteException if remote object is not available any more
	 */
	public Session login(String username, String pass) throws RemoteException {
        Session new_session = null;
        String userpath = ObjectManagerPaths.SESSION_MANAGER_USER_PATH;
        Accounts acs = new Accounts(userpath);//get user via UserHandler
        if (acs.exists(username) && acs.getProperty(username).compareTo(pass) == 0) {
        	User user =  new User(username, userpath);
        	username_map_user_.put(username, user);
        	new_session = SessionManager.getInstance().addSession(user);
        }
        if(logger.isInfoEnabled())
            logger.info("WA<-SM: login <- sessionid=" + new_session.getId());
        return new_session;
	}
	
	/**
	 * Logs the user out.
	 * This method removes the seession from all maps to allow the garbage 
	 * collector to destroy the object.
	 * 
	 * @param session the session to log out
	 * @throws RemoteException if remote object is not available any more
	 */	
	public void logout(Session session)throws RemoteException{
		String session_id = session.getId();
		SessionManager.getInstance().removeSession(session_id);
	
	}

	/**
	 * Returns the User object of the user belonging to the session
	 * with the specified ID. If no Session with this ID is
	 * known, null is returned.
	 * 
	 * @param session_id
	 * @return the User who owns the session or null if session_id is unknown
	 * @throws RemoteException if remote object is not available any more
	 */
	public User getUser(String session_id) throws RemoteException {
		String username = SessionManager.getInstance().getUserNameByID(session_id);
		if(username == null)
			return null;
        return username_map_user_.get(username);
	}

	/**
	 * Returns the username of the user belonging to the session
	 * with the specified ID as String. If no Session with this ID is
	 * known, null is returned.
	 * 
	 * @param session_id
	 * @return the username who owns the session or null if session_id is unknown
	 * @throws RemoteException if remote object is not available any more
	 */
	public String getUsername(String session_id) throws RemoteException {
		return SessionManager.getInstance().getUserNameByID(session_id);
	}

	/**
	 * returns the instance of the singleton UserManager. Since the
	 * constructor is private, this is the only method to get a
	 * UserManager. It always returns one and the same instance of
	 * this class.
	 * 
	 * @return the one and only instance of this class.
	 * @throws RemoteException if remote object is not available any more
	 */
	public static UserManager getInstance() throws RemoteException {
		if (instance_ == null) {
			try {
				instance_ = new UserManager();
			} catch (RemoteException e) {
				e.printStackTrace();
			}
		}
		return instance_;
	}

}
